# -*- coding: utf-8 -*-
import datetime

import mock
from mock import MagicMock, Mock, patch
from twisted.internet import defer, reactor
from twisted.internet.protocol import ClientFactory
from twisted.test import proto_helpers
from twisted.trial import unittest

from maranet.constructs.structs import MaraFrame
from maranet.protocols.client import MaraClientProtocol, MaraPorotocolFactory

from .mocks import COMasterMock

class BaseTestProtocol(unittest.TestCase):

    def setUp(self):
        self.comaster = COMasterMock()

def getStopPollInlineCalllback(stopping_action):
    """
    Utility function to stop client mainLoop
    """
    @defer.inlineCallbacks
    def stopPolling(*args, **kwargs):
        '''Generic inlineCallback that stop reactor'''
        stopping_action()
        yield ''
    return stopPolling

class TestProtocolPeh(BaseTestProtocol):
    @defer.inlineCallbacks
    def test_send_peh_upon_connection(self):
        '''To test client protocol we isloate it from the ClientFactory'''
        with patch.object(datetime, 'datetime', Mock(wraps=datetime.datetime)) as patched:
            fixed_date = datetime.datetime(2014, 1, 1, 12, 0, 0)
            patched.now.return_value = fixed_date

            factory = ClientFactory()
            factory.comaster = self.comaster

            factory.protocol = MaraClientProtocol
            proto = factory.buildProtocol(('127.0.0.1', 0))
            proto.construct = MaraFrame

            transport = proto_helpers.StringTransport()
            proto.makeConnection(transport)
            proto.doPoll = getStopPollInlineCalllback(transport.loseConnection)

            yield proto.mainLoop()
            bytes_sent_to_device = transport.value()
            assert len(bytes_sent_to_device)
            result = MaraFrame.parse(bytes_sent_to_device)
            self.assertEqual(result.dest, 0xFF)
            self.assertEqual(result.source, 2)

            # We don't need to check BCC since it's already coded into MaraFrame
            assert result.peh == fixed_date

class TestPrtocolFactory(unittest.TestCase):
    def setUp(self):
        self.comaster = COMasterMock()
        self.factory = MaraPorotocolFactory(self.comaster)

    # @mock.patch('maranet.mara.client.base.get_settings')
    def test_protocol_holds_reference_to_factory(self):
        protocol = self.factory.buildProtocol(('127.0.0.1', 0))
        self.assertEqual(protocol.factory, self.factory)

    def test_protocol_holds_reference_to_comaster(self):
        protocol = self.factory.buildProtocol(('127.0.0.1', 0))
        self.assertEqual(protocol.comaster, self.comaster)



import logging
from cStringIO import StringIO
from .mocks import COMasterMock
from maranet.protocols.loggers import COMasterLogAdapter
import unittest


class TestLogAdapter(unittest.TestCase):

    def setUp(self):
        self.stringio = StringIO()

        logger = logging.getLogger('')
        logger.setLevel(logging.DEBUG)
        # create console handler with a higher log level
        ch = logging.StreamHandler(stream=self.stringio)
        ch.setLevel(logging.DEBUG)
        # create formatter and add it to the handlers
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

        ch.setFormatter(formatter)
        # add the handlers to the logger
        logger.addHandler(ch)
        self.comaster = COMasterMock()
        self.adapter = COMasterLogAdapter(logger, {'comaster': self.comaster})

    def get_logged_lines(self):
        text = self.stringio.getvalue()
        lines = text.split('\n')
        lines = filter(len, lines)
        return lines

    def test_comaster_ip_present_in_output(self):

        self.adapter.debug('debug message')
        self.adapter.info('info message')
        self.adapter.warning('warn message')
        self.adapter.error('error message')
        self.adapter.critical('critical message')

        logged_lies = self.get_logged_lines()

        self.assertEqual(len(logged_lies), 5)

        for line in self.get_logged_lines():
            self.assertIn(self.comaster.ip_address, line)



from mock import MagicMock


def COMasterMock():
    '''Creates a COMaster mock, should reassemble to the model one.
    We don't use factory boy here because we don't want to generate interdependencies
    with Django ORM code. This mock should have the bare minimum constants nedded
    for a protocol/clientfactory to work.
    '''
    comaster = MagicMock()
    comaster.ip_address = '127.0.0.1'
    comaster.port = 9761
    comaster.sequence = None
    comaster.poll_interval = 5
    comaster.rs485_destination = 1
    comaster.rs485_source = 2
    return comaster



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    # Network lib
    'twisted',
    # Binary to Python objects and back
    'construct<=2.5.0',
    # Formats output of large events for human readable dumps
    'jinja2',
    # This is a Python lib
    'Click>=6.0',
]

setup_requirements = [
    'pytest-runner',
    'wheel',
    # TODO(D3f0): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    'pytest',
    'pdbpp',
    'mock'
    # TODO: put package test requirements here
]

def get_version():
    """Check code version"""
    import maranet
    return maranet.__version__

setup(
    name='maranet',
    version=get_version(),
    description="A Python client library for MARA protocol. Includes server emulator, although MARA servers are tipically microcontrollers.",
    long_description=readme + '\n\n' + history,
    author="Nahuel Defossé",
    author_email='ndefosse@tw.unp.edu.ar',
    url='https://github.com/D3f0/maranet',
    packages=find_packages(include=['maranet']),
    entry_points={
        'console_scripts': [
            'maranet=maranet.cli:cli'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='maranet',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)
